//----------------------------------------------------------------------------
//
// DockingApp: DockingEx toolbar example
//
//----------------------------------------------------------------------------
#include <owl/pch.h>
#include <owl/printer.h>
#include <owl/buttonga.h>
#include <owl/textgadg.h>
#include <owl/controlb.h>
#include <owl/controlg.h>
#include <owl/statusba.h>
#include <owl/docking.h>
#include <owl/combobox.h>
#include <owl/edit.h>
#include <owl/decmdifr.h>
#include <owl/editfile.h>
#include <owl/celarray.h>
#include <dir.h>
#include <owlext/pch.h>
#include <owlext/dockingex.h>
#include <owlext/gadgetex.h>
#include <owlext/gadgctrl.h>
#include <owlext/gadgctrl.h>
#include <owlext\toolcustdlg.h>
#include <owlext/butappdlg.h>

////////////////////////////////////////////////////////////////////////////////
#include <owlext/owlxlink.h>  // have to be only once
////////////////////////////////////////////////////////////////////////////////

#include "dockingapp.h"

#ifdef CTXHELP
//	#include <owlext/dockingex.rh>
#endif

#include <owlext/ctxhelpm.rh>

const MaxFontSizeLen = 20;
const MaxFontNameLen = 40;
const DefaultEditHeight = 20;
const MaxEditTextLen = 4;

DEFINE_RESPONSE_TABLE1(TMDIDecFrame, TDecoratedMDIFrame)
#ifdef BMPMENU
	EV_WM_INITMENU,
	EV_WM_INITMENUPOPUP,
	EV_WM_DRAWITEM,
	EV_WM_MEASUREITEM,
	EV_WM_MENUCHAR,
#endif
END_RESPONSE_TABLE;

#ifdef CTXHELP
DEFINE_HELPCONTEXT(TMDIDecFrame)
	HCENTRY_CONTROL(IDC_FONTNAME, IDOK),
	HCENTRY_CONTROL(IDC_FONTSIZE, IDCANCEL),
END_HELPCONTEXT;
#endif

TMDIDecFrame::TMDIDecFrame(const char far* title, TResId menuResId,
	TMDIClient &clientWnd, bool trackMenuSelection, TModule* module):
	TDecoratedMDIFrame(title, menuResId, clientWnd, trackMenuSelection, module)
{
#ifdef BMPMENU
	BmpMenu = new TBmpMenu(GetModule());

  // Set with accelerator always true, because PostGetHintText() of
	// THarborEx delete them if the user set them to to false
  //
  BmpMenu->SetWithAccel(true);
#endif
}


TMDIDecFrame::~TMDIDecFrame()
{
#ifdef BMPMENU
  delete BmpMenu;
#endif
}

#ifdef BMPMENU
bool TMDIDecFrame::SetMenu(HMENU hMenu)
{
  bool retVal = TDecoratedMDIFrame::SetMenu(hMenu);

  // Remap the main menu here one time, to get the right tooltips
  // from the beginning. If you don't do this, you have the right
  // tooltips at the first time then you click on a menu item ->EvInitMenu();
  //
  if (BmpMenu)
    BmpMenu->RemapMenu(hMenu);

  return retVal;
}
#endif

int TMDIDecFrame::GetHintText(uint id, char* buf, int size,
	THintText hintType) const
{
  int res;
#ifdef BMPMENU
  // Call BmpMenu->GetHintText() to get tooltips
  //
  if (BmpMenu)
  {
    res = BmpMenu->PreGetHintText(GetMenu(),
      id, buf, size, hintType);
    if (res == 0)
      res = TDecoratedMDIFrame::GetHintText(id, buf, size, hintType);
  }
#else
  res = TDecoratedMDIFrame::GetHintText(id, buf, size, hintType);
#endif

  // PostGetHintText from THarborEx() can
  // hiding tooltips and accelerators
  //
  THarborManagement* app = TYPESAFE_DOWNCAST(GetApplication(),
  	THarborManagement);
  if (app && app->GetHarbor())
    res = app->GetHarbor()->PostGetHintText(id, buf, size, hintType);
  return res;
}

LRESULT TMDIDecFrame::WindowProc(uint msg, WPARAM wParam, LPARAM lParam)
{
#ifdef CTXHELP
  TCtxHelpFileManager* ctxHelpM = TYPESAFE_DOWNCAST(GetApplication(),
  	TCtxHelpFileManager);
  if (ctxHelpM)
    if (ctxHelpM->PreProcessAppMsg(HWindow, msg, wParam, lParam))
      return true;
#endif
  return TDecoratedMDIFrame::WindowProc(msg, wParam, lParam);
}

#ifdef BMPMENU
void TMDIDecFrame::EvInitMenu(HMENU hMenu)
{
  // Remap the MDI application main menu to a bitmap menu
  //
  if (BmpMenu)
    BmpMenu->RemapMenu(hMenu);
  TDecoratedMDIFrame::EvInitMenu(hMenu);
}

void TMDIDecFrame::EvInitMenuPopup(HMENU hPopupMenu, uint index, bool sysMenu)
{
//  if (BmpMenu)
//    BmpMenu->RemapMenu(hPopupMenu);
  TDecoratedMDIFrame::EvInitMenuPopup(hPopupMenu, index, sysMenu);
}

void TMDIDecFrame::EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo)
{
  if (BmpMenu && BmpMenu->DrawItem(drawInfo))
    return;
  TDecoratedMDIFrame::EvDrawItem(ctrlId, drawInfo);
}

void TMDIDecFrame::EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo)
{
  if (BmpMenu && BmpMenu->MeasureItem(measureInfo))
    return;
	TDecoratedMDIFrame::EvMeasureItem(ctrlId, measureInfo);
}


int32 TMDIDecFrame::EvMenuChar(uint nChar, uint menuType, HMENU hMenu)
{
  LRESULT lRes;
  if (BmpMenu && BmpMenu->FindKeyboardShortcut(nChar, menuType, hMenu, lRes))
    return lRes;
  return TDecoratedMDIFrame::EvMenuChar(nChar, menuType, hMenu);
}


#endif

////////////////////////////////////////////////////////////////////////////////
TToolbarCustomizeSheet* 
TMyHarborEx::CreateCustomizeDlg(TDecoratedFrame& frame)
{
	return new TToolbarCustomizeSheet(&frame, this);
}
//
void
TMyHarborEx::ExecuteGadgetDialog(TDecoratedFrame& frame, TButtonTextGadgetEx* btg)
{
	TIntArray* array = new TIntArray;
	for(int i = 0; i < 49; i++)
		array->Add(i+FIRSTBITMAP);

	TCelArray* celarray = new TCelArray(TDib(*gModule, TResId(IDB_IMAGES)), 49);
	TButtonAppearanceDialog(&frame, btg, celarray, array).Execute();
}
//

//
// Build a response table for all messages/commands handled
// by the application.
//
#ifdef CTXHELP
DEFINE_RESPONSE_TABLE3(TDockingApp, TApplication, TCtxHelpFileManager,
	THarborManagement)
#else
DEFINE_RESPONSE_TABLE2(TDockingApp, TApplication, THarborManagement)
#endif
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_FILENEW, CmFileNew),
  EV_COMMAND(CM_FILEOPEN, CmFileOpen),
  EV_COMMAND(CM_FILEPRINT, CmFilePrint),
  EV_COMMAND(CM_FILEPRINTERSETUP, CmFilePrintSetup),
  EV_COMMAND(CM_FILEPRINTPREVIEW, CmFilePrintPreview),

  EV_COMMAND(CM_LOADTOOLBARS, CmLoadToolbars),
  EV_COMMAND_ENABLE(CM_LOADTOOLBARS, CeLoadToolbars),
  EV_COMMAND(CM_SAVETOOLBARS, CmSaveToolbars),

  EV_COMMAND(CM_ENABLECONTROLS, CmEnableControls),
  EV_COMMAND_ENABLE(CM_ENABLECONTROLS, CeEnableControls),

  EV_COMMAND(CM_LEVEL1, CmLevel1),
  EV_COMMAND(CM_LEVEL2, CmLevel2),
  EV_COMMAND(CM_LEVEL3, CmLevel3),

  EV_REGISTERED(TColorPickerMessage, CmColorPickerSelected),

  EV_COMMAND_ENABLE(CM_LEVELS, CeAlwaysTrue),
  EV_COMMAND_ENABLE(CM_LEVELS2, CeAlwaysTrue),
  EV_COMMAND_ENABLE(CM_TEXTCOLOR, CeAlwaysTrue),
  EV_COMMAND_ENABLE(CM_BKCOLOR, CeAlwaysTrue),
  EV_COMMAND_ENABLE(CM_LINECOLOR, CeAlwaysTrue),
  EV_COMMAND_ENABLE(CM_HILIGHTCOLOR, CeAlwaysTrue),
  EV_COMMAND_ENABLE(CM_FONTLEFT, CeAlwaysTrue),
  EV_COMMAND_ENABLE(CM_FONTCENTER, CeAlwaysTrue),
  EV_COMMAND_ENABLE(CM_FONTRIGHT, CeAlwaysTrue),

  EV_COMMAND(IDC_FONTNAME, CmFontName),
  EV_COMMAND(IDC_FONTSIZE, CmFontSize),

  EV_COMMAND(IDC_EDIT1, CmEdit1),
  EV_COMMAND(IDC_EDIT2, CmEdit2),
  EV_COMMAND(IDC_EDIT3, CmEdit3),
  EV_COMMAND(IDC_EDIT4, CmEdit4),

  EV_COMMAND_ENABLE(CM_FILEPRINT, CePrint),
  EV_COMMAND_ENABLE(CM_FILEPRINTERSETUP, CePrint),
  EV_COMMAND_ENABLE(CM_FILEPRINTPREVIEW, CePrint),
END_RESPONSE_TABLE;

//
//
//
TDockingApp::TDockingApp() :
  TApplication(_T("DockingApp")),
#ifdef CTXHELP
  TCtxHelpFileManager(),
#endif
 THarborManagement(_T("Software\\OWL NExt\\DockingApp"))
{
	// NOTE: This is a global variable
	//
	GadgetDescriptors = new TGadgetDescriptors();

	bEnableControls = true;
}

TDockingApp::~TDockingApp()
{
  delete GadgetDescriptors;
  if (::IsMenu(hLevelMenu))
		::DestroyMenu(hLevelMenu);
}

bool TDockingApp::CanClose()
{
  if (TApplication::CanClose()) {
    SaveSettings();
    return true;
  }
  return false;
}

void
TDockingApp::SetupSpeedBar(TDecoratedFrame* frame)
{
	Harbor = new TMyHarborEx(*frame);

	combo1 = new TGadgetComboBox(frame, IDC_FONTNAME, 0, 0, 180, 80,
		CBS_DROPDOWNLIST, MaxFontNameLen);
	combo2 = new TGadgetComboBox(frame, IDC_FONTSIZE, 0, 0, 50, 80,
		CBS_DROPDOWN, MaxFontSizeLen);
	edit1 = new TGadgetEdit(frame, IDC_EDIT1, TEXT(_T("10")), 0, 0, 60,
		DefaultEditHeight, MaxEditTextLen);
	edit2 = new TGadgetEdit(frame, IDC_EDIT2, TEXT(_T("20")), 0, 0, 60,
		DefaultEditHeight, MaxEditTextLen);
	edit3 = new TGadgetEdit(frame, IDC_EDIT3, TEXT(_T("30")), 0, 0, 60,
		DefaultEditHeight, MaxEditTextLen);
	edit4 = new TGadgetEdit(frame, IDC_EDIT4, TEXT(_T("40")), 0, 0, 60,
		DefaultEditHeight, MaxEditTextLen);

	GadgetDescriptors->Array.Add(new TControlGadgetDesc(*combo1));
	GadgetDescriptors->Array.Add(new TControlGadgetDesc(*combo2));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(0, Bitmap,
		CM_FONTBOLD, CM_FONTBOLD));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(0, Bitmap,
		CM_FONTITALIC, CM_FONTITALIC));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(0, Bitmap,
		CM_FONTUNDER, CM_FONTUNDER));

	// Note the Exclusive group will be set later
	//
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(0, Bitmap,
		CM_FONTLEFT, CM_FONTLEFT));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(0, Bitmap,
		CM_FONTCENTER, CM_FONTCENTER));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(0, Bitmap,
		CM_FONTRIGHT, CM_FONTRIGHT));

	// Select Automatic color that will be draw in black
	// 16 colors, 8 column, "Automatic" field, No "More colors" field
	//
	TColorPickerData cpd1(TColorPickerData::ColorTable16, 16, 8, false, true,
		IDS_AUTOMATIC, 0xFF000000, false);
	GadgetDescriptors->Array.Add(new TColorButtonGadgetDesc(DownArrowAction,
		cpd1, 0xFF000000, &TColorButtonGadgetEx::DefaultFillRect,
		frame, _T("Text Color"), Bitmap, CM_TEXTCOLOR, CM_TEXTCOLOR));

	// Select Automatic color that will be draw in white
	// 40 colors, 8 column, "Automatic" field, "More colors" field
	//
	TColorPickerData cpd2(TColorPickerData::ColorTable40, 40, 8, false, true,
		IDS_AUTOMATIC, 0xFFFFFFFF);
	GadgetDescriptors->Array.Add(new TColorButtonGadgetDesc(DownArrowAction,
		cpd2, 0xFFFFFFFF, &TColorButtonGadgetEx::DefaultFillRect,
		frame, _T("Background Color"), Bitmap, CM_BKCOLOR, CM_BKCOLOR));

	// Select Automatic color that will be draw in LtGray
	// 40 colors, 8 column, "None" field, "More colors" field
	//
	TColorPickerData cpd3(TColorPickerData::ColorTable40, 40, 8, false, true,
		IDS_NOCOLOR, 0xFFC0C0C0);
	GadgetDescriptors->Array.Add(new TColorButtonGadgetDesc(DownArrowAction,
		cpd3, 0xFFC0C0C0, &TColorButtonGadgetEx::DefaultFillRect,
		frame, _T("Line Color"), Bitmap, CM_LINECOLOR, CM_LINECOLOR));

	// Select Automatic color that will be draw in LtGray
	// 8 colors, 4 column, "None" field, No "More colors" field
	//
	TColorPickerData cpd4(TColorPickerData::ColorTable8, 8, 4, false, true,
		IDS_NOCOLOR, 0xFFC0C0C0, false);
	GadgetDescriptors->Array.Add(new TColorButtonGadgetDesc(DownArrowAction,
		cpd4, TColor::LtRed, &TColorButtonGadgetEx::DefaultFillRect,
		frame, _T("Hilight Color"), Bitmap, CM_HILIGHTCOLOR, CM_HILIGHTCOLOR));

	GadgetDescriptors->Array.Add(new TTextGadgetDesc(
		650, TGadget::None, TTextGadget::Left, 1, _T("X")));
	GadgetDescriptors->Array.Add(new TControlGadgetDesc(*edit1));
	GadgetDescriptors->Array.Add(new TTextGadgetDesc(
		660, TGadget::None, TTextGadget::Left, 1, _T("Y")));
	GadgetDescriptors->Array.Add(new TControlGadgetDesc(*edit2));
	GadgetDescriptors->Array.Add(new TTextGadgetDesc(
		670, TGadget::None, TTextGadget::Left, 1, _T("W")));
	GadgetDescriptors->Array.Add(new TControlGadgetDesc(*edit3));
	GadgetDescriptors->Array.Add(new TTextGadgetDesc(
		680, TGadget::None, TTextGadget::Left, 1, _T("H")));
	GadgetDescriptors->Array.Add(new TControlGadgetDesc(*edit4));

	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("New"), Bitmap, CM_FILENEW, CM_FILENEW));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Open"), Bitmap, CM_FILEOPEN, CM_FILEOPEN));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Save"), Bitmap, CM_FILESAVE, CM_FILESAVE));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Cut"), Bitmap, CM_EDITCUT, CM_EDITCUT));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Copy"), Bitmap, CM_EDITCOPY, CM_EDITCOPY));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Paste"), Bitmap, CM_EDITPASTE, CM_EDITPASTE));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Undo"), Bitmap, CM_EDITUNDO, CM_EDITUNDO));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Redo"), Bitmap, CM_EDITREDO, CM_EDITREDO));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Find"), Bitmap, CM_EDITFIND, CM_EDITFIND));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Next"), Bitmap, CM_EDITFINDNEXT, CM_EDITFINDNEXT));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Print"), Bitmap, CM_FILEPRINT, CM_FILEPRINT));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Preview"), Bitmap, CM_FILEPRINTPREVIEW, CM_FILEPRINTPREVIEW));

	GadgetDescriptors->Array.Add(new TBitmapGadgetDesc(
		IDB_ROTATE, IDB_ROTATE, TGadget::Recessed, 1));

#ifdef CTXHELP
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Contents"), Bitmap, CM_HELPCONTENTS, CM_HELPCONTENTS));
	GadgetDescriptors->Array.Add(new TButtonTextGadgetDesc(
		_T("Contexthelp"), Bitmap, CM_HELPCONTEXT, CM_HELPCONTEXT));
#endif

	// Insert descriptors for Exclusive and SemiExclusive button gadget groups
	//
	TButtonGadgetGroup* group = new TButtonGadgetGroup(TButtonGadget::Exclusive);
	group->Array.Add(CM_FONTLEFT);
	group->Array.Add(CM_FONTCENTER);
	group->Array.Add(CM_FONTRIGHT);
	GadgetDescriptors->Groups.Add(group);

	// Insert descriptors for command categories
	//
	TCommandCategory* category1 = new TCommandCategory(_T("File"));
	category1->Array.Add(CM_FILENEW);
	category1->Array.Add(CM_FILEOPEN);
	category1->Array.Add(CM_FILESAVE);
	category1->Array.Add(CM_FILEPRINT);
	category1->Array.Add(CM_FILEPRINTPREVIEW);
	GadgetDescriptors->Categories.Add(category1);

	TCommandCategory* category2 = new TCommandCategory(_T("Edit"));
	category2->Array.Add(CM_EDITCUT);
	category2->Array.Add(CM_EDITCOPY);
	category2->Array.Add(CM_EDITPASTE);
	category2->Array.Add(CM_EDITUNDO);
	category2->Array.Add(CM_EDITREDO);
	category2->Array.Add(CM_EDITFIND);
  category2->Array.Add(CM_EDITFINDNEXT);
  GadgetDescriptors->Categories.Add(category2);

  TCommandCategory* category3 = new TCommandCategory(_T("Format"));
  category3->Array.Add(CM_FONTBOLD);
  category3->Array.Add(CM_FONTITALIC);
  category3->Array.Add(CM_FONTUNDER);
	category3->Array.Add(CM_FONTLEFT);
  category3->Array.Add(CM_FONTCENTER);
	category3->Array.Add(CM_FONTRIGHT);
  category3->Array.Add(CM_TEXTCOLOR);
	category3->Array.Add(CM_BKCOLOR);
  category3->Array.Add(CM_LINECOLOR);
  category3->Array.Add(CM_HILIGHTCOLOR);
  GadgetDescriptors->Categories.Add(category3);

  TCommandCategory* category4 = new TCommandCategory(_T("Help"));
  category4->Array.Add(CM_HELPCONTENTS);
  category4->Array.Add(CM_HELPCONTEXT);
  GadgetDescriptors->Categories.Add(category4);

  // IMPORTANT: At the end of all descriptor definitions check consistence
  //
  GadgetDescriptors->CheckConsistence();

  // Create text style toolbar and associate toolbar buttons with commands.
  //

	TDockableControlBarEx* MyToolbar1 = new TDockableControlBarEx(1,
		_T("Toolbar with Combobox"), frame);
	int gadIds1[19] = {
		IDC_FONTNAME, 0, IDC_FONTSIZE, 0, CM_FONTBOLD, CM_FONTITALIC, CM_FONTUNDER,
		0, CM_FONTLEFT, CM_FONTCENTER, CM_FONTRIGHT, 0, CM_LEVELS, CM_LEVELS2, 0,
		CM_TEXTCOLOR, CM_BKCOLOR, CM_LINECOLOR, CM_HILIGHTCOLOR };
	Harbor->Insert(*MyToolbar1, alTop);
	MyToolbar1->SetDefaultGadgets(gadIds1, 19);

	// Insert the default toolbar, that support help
	//
	TDockableControlBarEx* MyToolbar2 = new TDockableControlBarEx(IDW_TOOLBAR,
		_T("Default Toolbar"), frame, true, true);
#ifdef CTXHELP
	int gadIds2[19] = {
		CM_FILENEW, CM_FILEOPEN, CM_FILESAVE, 0, CM_EDITCUT, CM_EDITCOPY,
		CM_EDITPASTE, 0, CM_EDITUNDO, 0, CM_EDITREDO, 0, CM_EDITFIND,
		CM_EDITFINDNEXT, 0, CM_FILEPRINT, CM_FILEPRINTPREVIEW, CM_HELPCONTENTS,
		CM_HELPCONTEXT};
	Harbor->Insert(*MyToolbar2, alTop);
	MyToolbar2->SetDefaultGadgets(gadIds2, 19);
#else
	int gadIds2[17] = {
		CM_FILENEW, CM_FILEOPEN, CM_FILESAVE, 0, CM_EDITCUT, CM_EDITCOPY,
		CM_EDITPASTE, 0, CM_EDITUNDO, 0, CM_EDITREDO, 0, CM_EDITFIND,
		CM_EDITFINDNEXT, 0, CM_FILEPRINT, CM_FILEPRINTPREVIEW};
	Harbor->Insert(*MyToolbar2, alTop);
	MyToolbar2->SetDefaultGadgets(gadIds2, 17);
#endif

	// Insert a copy of default
	//

	TDockableControlBarEx* MyToolbar3 = new TDockableControlBarEx(3,
		_T("Copy Default Toolbar"), frame);
#ifdef CTXHELP
	Harbor->Insert(*MyToolbar3, alRight);
	MyToolbar3->SetDefaultGadgets(gadIds2, 19);
#else
	Harbor->Insert(*MyToolbar3, alRight);
	MyToolbar3->SetDefaultGadgets(gadIds2, 17);
#endif

	// Create coord toolbar and associate toolbar buttons with commands.
	//

	TDockableControlBarEx* MyToolbar4 = new TDockableControlBarEx(4,
		_T("Toolbar with Edit controls"), frame);
	int gadIds4[13] = {
		IDB_ROTATE, 0, 650, IDC_EDIT1, 0, 660, IDC_EDIT2, 0, 670, IDC_EDIT3, 0,
			680, IDC_EDIT4 };
	Harbor->Insert(*MyToolbar4, alLeft);
	MyToolbar4->SetDefaultGadgets(gadIds4, 13);

}

//
// Application intialization.
//
void TDockingApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  MdiClient = new TMDIClient;
  TMDIDecFrame* frame = new TMDIDecFrame(Name, IDM_MAIN, *MdiClient, true);

  // Assign ICON w/ this application.
  //
  frame->SetIcon(this, IDI_MDIAPPLICATION);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDA_MAIN;

  SetupSpeedBar(frame);

#ifdef CTXHELP
  // Use TCtxHelpStatusBar instead of TStatusBar
  TCtxHelpStatusBar* sb = new TCtxHelpStatusBar(frame, TGadget::Recessed,
#else
  TStatusBar* sb = new TStatusBar(frame, TGadget::Recessed,
#endif
                                  TStatusBar::CapsLock |
                                  TStatusBar::NumLock |
                                  TStatusBar::ScrollLock |
                                  TStatusBar::SizeGrip);

  // Show hints in the text gadget
  //
  sb->SetWideHints(false);
  frame->Insert(*sb, TDecoratedFrame::Bottom);

	SetMainWindow(frame);
	frame->SetMenuDescr(TMenuDescr(IDM_MAIN, 1, 2, 0, 0, 1, 1));
}

void TDockingApp::InitInstance()
{
	TApplication::InitInstance();

	combo1->InsertString(_T("Arial"), 0);
	combo1->InsertString(_T("Courier New"), 1);
	combo1->InsertString(_T("Times New Roman"), 2);
	combo1->SetSelIndex(0);
	combo2->InsertString(_T("8"), 0);
	combo2->InsertString(_T("10"), 1);
	combo2->InsertString(_T("12"), 2);
	combo2->SetText(_T("8"));
	oldSelFontSize = _T("8");

	hLevelMenu = LoadMenu(IDM_LEVELS);
	if (hLevelMenu && ::IsMenu(hLevelMenu) &&
			::GetSubMenu(hLevelMenu, 0) && ::IsMenu(::GetSubMenu(hLevelMenu, 0))) {
		GadgetDescriptors->Array.Add(new TMenuButtonGadgetDesc(DownArrow,
			::GetSubMenu(hLevelMenu, 0), GetMainWindow(), _T("Levels1"), Text,
			CM_LEVELS, CM_LEVELS));
		GadgetDescriptors->Array.Add(new TMenuButtonGadgetDesc(Normal,
			::GetSubMenu(hLevelMenu, 0), GetMainWindow(), _T("Levels2"), Text,
			CM_LEVELS, CM_LEVELS2));
		TCommandCategory* cc = GadgetDescriptors->FindCategory(string(_T("Format")));
		if (cc) {
			cc->Array.Add(CM_LEVELS);
			cc->Array.Add(CM_LEVELS2);
			GadgetDescriptors->CheckConsistence(); // For safty
		}
	}

	LoadSettings();
}

void TDockingApp::CmFileNew()
{
	char  title[255];

	// Generate a title for the MDI child window.
	//
	wsprintf(title, _T("%d"), ChildCount++);

	TMDIChild* child = new TMDIChild(*MdiClient, title, new TEditFile(0, 0, 0));

  // Associate ICON w/ this child window.
  //
  child->SetIcon(this, IDI_DOC);

  // If the current active MDI child is maximize then this one should be also.
  //
  TMDIChild* curChild = MdiClient->GetActiveMDIChild();
  if (curChild && (curChild->GetWindowLong(GWL_STYLE) & WS_MAXIMIZE))
    child->Attr.Style |= WS_MAXIMIZE;

  child->Create();
}

void TDockingApp::OpenFile(const char* fileName)
{
  if (fileName)
    strcpy(FileData.FileName, fileName);

  // Create a MDIChild window whose client is TEditFile.
  //
  TMDIChild* child = new TMDIChild(*MdiClient, _T(""),
    new TEditFile(0, 0, 0, 0, 0, 0, 0, FileData.FileName));

  // Associate ICON w/ this child window.
  //
  child->SetIcon(this, IDI_DOC);

  // If the current active MDI child is maximize then this one should be also.
  //
  TMDIChild* curChild = MdiClient->GetActiveMDIChild();
  if (curChild && (curChild->GetWindowLong(GWL_STYLE) & WS_MAXIMIZE))
    child->Attr.Style |= WS_MAXIMIZE;

  child->Create();
}

//
// Menu File Open command
//
void TDockingApp::CmFileOpen()
{
  // Display standard Open dialog box to select a file name.
  //
  *FileData.FileName = 0;
  if (TFileOpenDialog(GetMainWindow(), FileData).Execute() == IDOK)
    OpenFile();
}

//
// Menu File Print command
//
void TDockingApp::CmFilePrint()
{
}

//
// Menu File Print Setup command
//
void TDockingApp::CmFilePrintSetup()
{
}

//
// Menu File Print Preview command
//
void TDockingApp::CmFilePrintPreview()
{
}

//
// Menu enabler used by Print, Print Setup and Print Preview.
//
void TDockingApp::CePrint(TCommandEnabler& ce)
{
  ce.Enable(true);
}

void TDockingApp::CmLoadToolbars()
{
  LoadSettings();
}

void TDockingApp::CeLoadToolbars(TCommandEnabler& ce)
{
  InsertToolbarMenuItem(ce, 1);
#ifdef BMPMENU
  // remap the inserted menu to a bitmap menu
  //
  TMenuItemEnabler* me = TYPESAFE_DOWNCAST(&ce, TMenuItemEnabler);
  if (me && me->GetMenu())
  {
    TMDIDecFrame* frame =
      TYPESAFE_DOWNCAST(GetMainWindow(), TMDIDecFrame);
    if (frame && frame->GetBmpMenu())
      frame->GetBmpMenu()->RemapMenu(me->GetMenu(), true);
  }
#endif
}

void TDockingApp::CmSaveToolbars()
{
  SaveSettings();
}

void TDockingApp::CmEnableControls()
{
	bEnableControls = bEnableControls ? false : true;
  combo1->EnableWindow(bEnableControls);
  combo2->EnableWindow(bEnableControls);
  edit1->EnableWindow(bEnableControls);
  edit2->EnableWindow(bEnableControls);
  edit3->EnableWindow(bEnableControls);
  edit4->EnableWindow(bEnableControls);
}

void TDockingApp::CeEnableControls(TCommandEnabler& ce)
{
  ce.SetCheck(bEnableControls);
}

void TDockingApp::CeAlwaysTrue(TCommandEnabler& ce)
{
  ce.Enable(true);
}

void TDockingApp::CmLevel1()
{
  GetMainWindow()->MessageBox(_T("Level 1 pressed"), 0, MB_OK);
}

void TDockingApp::CmLevel2()
{
  GetMainWindow()->MessageBox(_T("Level 2 pressed"), 0, MB_OK);
}

void TDockingApp::CmLevel3()
{
  GetMainWindow()->MessageBox(_T("Level 3 pressed"), 0, MB_OK);
}

int32 TDockingApp::CmColorPickerSelected(TParam1 param1, TParam2 param2)
{
  char buf[255];
  TColor color = TColor((COLORREF) param2);
  uint8 flags = color.Flags();
  switch (param1) {
		case CM_TEXTCOLOR:
  	  if (flags == 0)
    	  wsprintf(buf, _T("Text Color changed: RGB(%u, %u, %u)"),
        	color.Red(), color.Green(), color.Blue());
	    else
  	    wsprintf(buf, _T("Text Color changed: Select the Default Color"));
      break;

    case CM_BKCOLOR:
  	  if (flags == 0)
				wsprintf(buf, _T("Background Color changed: RGB(%u, %u, %u)"),
        	color.Red(), color.Green(), color.Blue());
	    else
  	    wsprintf(buf, _T("Background Color changed: Select the Default Color"));
      break;

    case CM_LINECOLOR:
  	  if (flags == 0)
    	  wsprintf(buf, _T("Line Color changed: RGB(%u, %u, %u)"),
        	color.Red(), color.Green(), color.Blue());
	    else
  	    wsprintf(buf, _T("Line Color changed: Select No Color"));
      break;

    case CM_HILIGHTCOLOR:
  	  if (flags == 0)
    	  wsprintf(buf, _T("Hilight Color changed: RGB(%u, %u, %u)"),
        	color.Red(), color.Green(), color.Blue());
	    else
  	    wsprintf(buf, _T("Hilight Color changed: Select No Color"));
      break;

  }
  GetMainWindow()->MessageBox(buf, _T("Color Picker Notification"), MB_OK);
  return 0;
}

//
// Menu Help About palette.exe command
//
void TDockingApp::CmHelpAbout()
{
  GetMainWindow()->MessageBox(_T("DockingApp: DockingEx toolbar example\n"
                                 "Copyright 1998 by Dieter Windau."
                                 " All Rights reserverd"), _T("About"), MB_OK);
}

void TDockingApp::CmFontName()
{
  if (oldSelFontName != combo1->GetSelIndex()) {
		oldSelFontName = combo1->GetSelIndex();
	  switch (combo1->GetSelIndex()) {
			case 0: GetMainWindow()->MessageBox(_T("Font index 0"),
      					_T("Font Name changed"), MB_OK);
              break;
			case 1: GetMainWindow()->MessageBox(_T("Font index 1"),
      					_T("Font Name changed"), MB_OK);
              break;
			case 2: GetMainWindow()->MessageBox(_T("Font index 2"),
      					_T("Font Name changed"), MB_OK);
              break;
  	}
  }
}

void TDockingApp::CmFontSize()
{
  TCHAR FontSizeStr[MaxFontSizeLen];
  combo2->GetText(FontSizeStr, MaxFontSizeLen);
  if (oldSelFontSize != string(FontSizeStr)) {
    oldSelFontSize = FontSizeStr;
    GetMainWindow()->MessageBox(FontSizeStr, _T("Font Size changed"), MB_OK);
  }
}

void TDockingApp::CmEdit1()
{
  TCHAR str[MaxEditTextLen];
  edit1->GetWindowText(str, MaxEditTextLen);
	GetMainWindow()->MessageBox(str, _T("Edit 1"), MB_OK);
}

void TDockingApp::CmEdit2()
{
  TCHAR str[MaxEditTextLen];
  edit2->GetWindowText(str, MaxEditTextLen);
	GetMainWindow()->MessageBox(str, _T("Edit 2"), MB_OK);
}

void TDockingApp::CmEdit3()
{
  TCHAR str[MaxEditTextLen];
  edit3->GetWindowText(str, MaxEditTextLen);
	GetMainWindow()->MessageBox(str, _T("Edit 3"), MB_OK);
}

void TDockingApp::CmEdit4()
{
  TCHAR str[MaxEditTextLen];
  edit4->GetWindowText(str, MaxEditTextLen);
	GetMainWindow()->MessageBox(str, _T("Edit 4"), MB_OK);
}

//
// Process application messages to provide context sensitive help
//
bool TDockingApp::ProcessAppMsg(MSG& msg)
{
#ifdef CTXHELP
  if (TCtxHelpFileManager::PreProcessAppMsg(msg))
    return true;
#endif

  if (Harbor && Harbor->PreProcessAppMsg(msg))
    return true;

  // Continue normal processing.
  //
  return TApplication::ProcessAppMsg(msg);
}

void TDockingApp::NeedForOWL6()
{
}

// bun in Borland
static void NeverUsedDummyFunction()
{
	TIntArray array;
	TObjectArray<int> test2;
	TIPtrArray<TCommandCategory*> test5;
	TIPtrArray<TButtonGadgetGroup*> test6;
	TIPtrArray<TGadgetDesc*> test7;
	TIPtrArray<TDCBData*> test8;
}
//
int OwlMain(int , char* [])
{
	return TDockingApp().Run();
}
//
