//----------------------------------------------------------------------------
//
// DockingApp: DockingEx toolbar example
//
//----------------------------------------------------------------------------
#include <owl/opensave.h>

#ifndef __OWLEXT_HARBOREX_H
#include <owlext/harborex.h>
#endif
#ifndef __OWLEXT_DOCKINGEX_H
#include <owlext/dockingex.h>
#endif

#ifdef CTXHELP
#include <owlext/ctxhelpm.h>
#endif

#ifdef BMPMENU
#include <owlext/bmpmenu.h>
#endif

#include "dockingapp.rh" // Definition of all resources.

// ****************** Forward declaration *************************************

class TGadgetComboBox;
class TGadgetEdit;

// ****************** TMDIDecFrame ********************************************

class TMDIDecFrame : public TDecoratedMDIFrame {
  public:
    TMDIDecFrame(const char far* title, TResId menuResId,
    	TMDIClient &clientWnd = *new TMDIClient, bool trackMenuSelection = false,
      TModule* module = 0);
    ~TMDIDecFrame();

  public:
#ifdef BMPMENU
    TBmpMenu* GetBmpMenu() { return BmpMenu; }
    virtual bool SetMenu(HMENU hMenu);
#endif

    int     GetHintText(uint id, char* buf, int size, THintText hintType) const;

  protected:
    LRESULT WindowProc(uint msg, WPARAM wParam, LPARAM lParam);

#ifdef BMPMENU
    void  EvInitMenu(HMENU hMenu);
    void  EvInitMenuPopup(HMENU hPopupMenu, uint index, bool sysMenu);
    void  EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo);
    void  EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo);


    int32 EvMenuChar(uint nChar, uint menuType, HMENU hMenu);

  protected:
    TBmpMenu* BmpMenu;
#endif

  DECLARE_RESPONSE_TABLE(TMDIDecFrame);
#ifdef CTXHELP
  DECLARE_HELPCONTEXT(TMDIDecFrame);
#endif
};

//------------------------------------------------------------------------------
class TMyHarborEx : public THarborEx{
	public:
		TMyHarborEx(TDecoratedFrame& df):THarborEx(df){}

		TToolbarCustomizeSheet* CreateCustomizeDlg(TDecoratedFrame& frame);
		void ExecuteGadgetDialog(TDecoratedFrame& frame, TButtonTextGadgetEx* btg);
};
// ****************** TDockingApp *********************************************

#ifdef CTXHELP
class TDockingApp : public TApplication, public TCtxHelpFileManager,
	public THarborManagement {
#else
class TDockingApp : public TApplication, public THarborManagement {
#endif
  public:
    TDockingApp();
   ~TDockingApp();

    bool CanClose();

    void NeedForOWL6();

    TOpenSaveDialog::TData FileData; // Data to control open/saveas dialog.

    TMDIClient* MdiClient;
    int         ChildCount;          // Number of child window created.

  protected:

    virtual void InitMainWindow();
    virtual void InitInstance();

    void OpenFile(const char* fileName = 0);

    void CmFileNew();
    void CmFileOpen();
    void CmFilePrint();
    void CmFilePrintSetup();
    void CmFilePrintPreview();
    void CePrint(TCommandEnabler& ce);

    void CmLoadToolbars();
    void CeLoadToolbars(TCommandEnabler& ce);
    void CmSaveToolbars();

    void CmEnableControls();
    void CeEnableControls(TCommandEnabler& ce);

		void CeAlwaysTrue(TCommandEnabler& ce);

		void CmLevel1();
		void CmLevel2();
		void CmLevel3();
    int32 CmColorPickerSelected(TParam1 param1, TParam2 param2);

    void CmHelpAbout();

    void CmFontName();
    void CmFontSize();

		void CmEdit1();
		void CmEdit2();
		void CmEdit3();
		void CmEdit4();

  private:
    void SetupSpeedBar(TDecoratedFrame* frame);
    void AddFiles(TFileDropletList& files);

    bool ProcessAppMsg(MSG& msg);

  protected:
    TGadgetComboBox* combo1;
    TGadgetComboBox* combo2;
    TGadgetEdit*     edit1;
    TGadgetEdit*     edit2;
    TGadgetEdit*     edit3;
    TGadgetEdit*     edit4;

    int        oldSelFontName;
    string     oldSelFontSize;

    bool       bEnableControls;
    HMENU      hLevelMenu;

  DECLARE_RESPONSE_TABLE(TDockingApp);
};
